/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 *  
 */
// https://issues.jboss.org/browse/TEIID-5515
package org.teiid.adminapi.impl;

import static org.junit.Assert.*;

import org.jboss.dmr.ModelNode;
import org.junit.Test;
import org.teiid.adminapi.Request.ProcessingState;
import org.teiid.adminapi.Request.ThreadState;
@SuppressWarnings("nls")
public class TestRequestMetadata {
	
	@Test public void testMapping() {
		RequestMetadata request = buildRequest();
		
		ModelNode node = VDBMetadataMapper.RequestMetadataMapper.INSTANCE.wrap(request, new ModelNode());
		
		RequestMetadata actual = VDBMetadataMapper.RequestMetadataMapper.INSTANCE.unwrap(node);
		
		assertEquals(request, actual);
		assertEquals(request.getState(), actual.getState());
	}

	private RequestMetadata buildRequest() {
		RequestMetadata request = new RequestMetadata();
		request.setState(ProcessingState.PROCESSING);
		request.setCommand("select * from foo"); //$NON-NLS-1$
		request.setExecutionId(1234);
		request.setName("request-name"); //$NON-NLS-1$
		request.setSessionId("session-id");//$NON-NLS-1$
		request.setSourceRequest(false);
		request.setStartTime(12345L);
		request.setTransactionId("transaction-id");//$NON-NLS-1$
		request.setThreadState(ThreadState.RUNNING);
		//request.setNodeId(1);
		return request;
	}

	public static final String desc = "{\"attributes\" : {\n" +
			"    \"execution-id\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"LONG\"\n" +
			"        },\n" +
			"        \"description\" : \"execution-id\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : true,\n" +
			"        \"nillable\" : false\n" +
			"    },\n" +
			"    \"session-id\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"STRING\"\n" +
			"        },\n" +
			"        \"description\" : \"session-id\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : true,\n" +
			"        \"nillable\" : false,\n" +
			"        \"min-length\" : 1,\n" +
			"        \"max-length\" : 2147483647\n" +
			"    },\n" +
			"    \"start-time\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"LONG\"\n" +
			"        },\n" +
			"        \"description\" : \"start-time\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : true,\n" +
			"        \"nillable\" : false\n" +
			"    },\n" +
			"    \"command\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"STRING\"\n" +
			"        },\n" +
			"        \"description\" : \"command\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : true,\n" +
			"        \"nillable\" : false,\n" +
			"        \"min-length\" : 1,\n" +
			"        \"max-length\" : 2147483647\n" +
			"    },\n" +
			"    \"source-request\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"BOOLEAN\"\n" +
			"        },\n" +
			"        \"description\" : \"source-request\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : true,\n" +
			"        \"nillable\" : false\n" +
			"    },\n" +
			"    \"node-id\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"INT\"\n" +
			"        },\n" +
			"        \"description\" : \"node-id\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : false,\n" +
			"        \"nillable\" : true\n" +
			"    },\n" +
			"    \"transaction-id\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"STRING\"\n" +
			"        },\n" +
			"        \"description\" : \"transaction-id\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : false,\n" +
			"        \"nillable\" : true,\n" +
			"        \"min-length\" : 1,\n" +
			"        \"max-length\" : 2147483647\n" +
			"    },\n" +
			"    \"processing-state\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"STRING\"\n" +
			"        },\n" +
			"        \"description\" : \"processing-state\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : true,\n" +
			"        \"nillable\" : false,\n" +
			"        \"min-length\" : 1,\n" +
			"        \"max-length\" : 2147483647\n" +
			"    },\n" +
			"    \"thread-state\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"STRING\"\n" +
			"        },\n" +
			"        \"description\" : \"thread-state\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : true,\n" +
			"        \"nillable\" : false,\n" +
			"        \"min-length\" : 1,\n" +
			"        \"max-length\" : 2147483647\n" +
			"    }\n" +
			"}}";
	@Test public void testDescribe() {
		assertEquals(desc, TestVDBMetaData.describe(new ModelNode(), VDBMetadataMapper.RequestMetadataMapper.INSTANCE.getAttributeDefinitions()).toJSONString(false));
	}

}
