/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 *  
 */
// https://issues.jboss.org/browse/TEIID-5515
package org.teiid.adminapi.impl;

import static org.junit.Assert.*;

import org.jboss.dmr.ModelNode;
import org.junit.Test;

@SuppressWarnings("nls")
public class TestSessionMetadata {
	
	@Test public void testMapping() {
		SessionMetadata session = new SessionMetadata();
		session.setSessionId("test");
		session.setApplicationName("foo");
		session.setClientHostName("localhost");
		session.setCreatedTime(1234);
		session.setIPAddress("127.0.0.1");
		session.setVDBName("vdb-name");
		session.setVDBVersion(2);
		session.setSecurityContext("auth-domain");
		session.setUserName("user");

		ModelNode node = VDBMetadataMapper.SessionMetadataMapper.INSTANCE.wrap(session, new ModelNode());
		
		SessionMetadata session1 = VDBMetadataMapper.SessionMetadataMapper.INSTANCE.unwrap(node);
		
		assertEquals(session.getSessionId(), session1.getSessionId());
		assertEquals(session.getApplicationName(), session1.getApplicationName());
		
	}

	private static final String describe = "{\"attributes\" : {\n" +
			"    \"application-name\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"STRING\"\n" +
			"        },\n" +
			"        \"description\" : \"application-name\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : false,\n" +
			"        \"nillable\" : true,\n" +
			"        \"min-length\" : 1,\n" +
			"        \"max-length\" : 2147483647\n" +
			"    },\n" +
			"    \"created-time\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"LONG\"\n" +
			"        },\n" +
			"        \"description\" : \"created-time\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : true,\n" +
			"        \"nillable\" : false\n" +
			"    },\n" +
			"    \"client-host-address\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"STRING\"\n" +
			"        },\n" +
			"        \"description\" : \"client-host-address\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : false,\n" +
			"        \"nillable\" : true,\n" +
			"        \"min-length\" : 1,\n" +
			"        \"max-length\" : 2147483647\n" +
			"    },\n" +
			"    \"client-hardware-address\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"STRING\"\n" +
			"        },\n" +
			"        \"description\" : \"client-hardware-address\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : false,\n" +
			"        \"nillable\" : true,\n" +
			"        \"min-length\" : 1,\n" +
			"        \"max-length\" : 2147483647\n" +
			"    },\n" +
			"    \"ip-address\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"STRING\"\n" +
			"        },\n" +
			"        \"description\" : \"ip-address\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : false,\n" +
			"        \"nillable\" : true,\n" +
			"        \"min-length\" : 1,\n" +
			"        \"max-length\" : 2147483647\n" +
			"    },\n" +
			"    \"last-ping-time\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"LONG\"\n" +
			"        },\n" +
			"        \"description\" : \"last-ping-time\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : true,\n" +
			"        \"nillable\" : false\n" +
			"    },\n" +
			"    \"session-id\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"STRING\"\n" +
			"        },\n" +
			"        \"description\" : \"session-id\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : true,\n" +
			"        \"nillable\" : false,\n" +
			"        \"min-length\" : 1,\n" +
			"        \"max-length\" : 2147483647\n" +
			"    },\n" +
			"    \"user-name\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"STRING\"\n" +
			"        },\n" +
			"        \"description\" : \"user-name\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : true,\n" +
			"        \"nillable\" : false,\n" +
			"        \"min-length\" : 1,\n" +
			"        \"max-length\" : 2147483647\n" +
			"    },\n" +
			"    \"vdb-name\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"STRING\"\n" +
			"        },\n" +
			"        \"description\" : \"vdb-name\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : true,\n" +
			"        \"nillable\" : false,\n" +
			"        \"min-length\" : 1,\n" +
			"        \"max-length\" : 2147483647\n" +
			"    },\n" +
			"    \"vdb-version\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"STRING\"\n" +
			"        },\n" +
			"        \"description\" : \"vdb-version\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : true,\n" +
			"        \"nillable\" : false,\n" +
			"        \"min-length\" : 1,\n" +
			"        \"max-length\" : 2147483647\n" +
			"    },\n" +
			"    \"security-domain\" : {\n" +
			"        \"type\" : {\n" +
			"            \"TYPE_MODEL_VALUE\" : \"STRING\"\n" +
			"        },\n" +
			"        \"description\" : \"security-domain\",\n" +
			"        \"expressions-allowed\" : false,\n" +
			"        \"required\" : false,\n" +
			"        \"nillable\" : true,\n" +
			"        \"min-length\" : 1,\n" +
			"        \"max-length\" : 2147483647\n" +
			"    }\n" +
			"}}";
	
	@Test public void testDescribe() {
		ModelNode n = TestVDBMetaData.describe(new ModelNode(), VDBMetadataMapper.SessionMetadataMapper.INSTANCE.getAttributeDefinitions());
		//System.out.println(n.toJSONString(false));
		assertEquals(describe, n.toJSONString(false));
	}
}
