/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 *  
 */
 
 package org.teiid.language;

import java.util.List;

import org.teiid.core.util.EquivalenceUtil;
import org.teiid.core.util.HashCodeUtil;
import org.teiid.language.visitor.LanguageObjectVisitor;

public class WindowSpecification extends BaseLanguageObject {
	
	private List<Expression> partition;
	private OrderBy orderBy;

	public WindowSpecification() {
		
	}
	
	public List<Expression> getPartition() {
		return partition;
	}
	
	public void setPartition(List<Expression> grouping) {
		this.partition = grouping;
	}
	
	public OrderBy getOrderBy() {
		return orderBy;
	}
	
	public void setOrderBy(OrderBy orderBy) {
		this.orderBy = orderBy;
	}

	@Override
	public void acceptVisitor(LanguageObjectVisitor visitor) {
		visitor.visit(this);
	}
	
	@Override
	public int hashCode() {
		return HashCodeUtil.hashCode(partition.hashCode(), orderBy);
	}
	
	public boolean equals(Object obj) {
		if (obj == this) {
			return true;
		}
		if (!(obj instanceof WindowSpecification)) {
			return false;
		}
		WindowSpecification other = (WindowSpecification)obj;
		return EquivalenceUtil.areEqual(this.partition, other.partition) &&
		EquivalenceUtil.areEqual(this.orderBy, other.orderBy);
	}
	
	// EXTENSION
	// ISSUE: DV-84
	// FIXED BY: Roman Pedchenko
	// DATE: 2013.06.24
	private Frame frame;
	public Frame getFrame() {
		return frame;
	}

	public void setFrame(Frame frame) {
		this.frame = frame;
	}

	public static enum FrameType { NONE, UNBND, ROW, EXPR } 
	public static class Frame extends BaseLanguageObject {
	    private boolean range;
	    private int startExpr;
	    private int endExpr;
	    private FrameType startType = FrameType.NONE;
	    private FrameType endType = FrameType.NONE;
	    private boolean startFollowing;
	    private boolean endFollowing;

		public boolean isRange() {
			return range;
		}

		public void setRange(boolean range) {
			this.range = range;
		}

		public int getStartExpr() {
			return startExpr;
		}

		public void setStartExpr(int startExpr) {
			this.startExpr = startExpr;
		}

		public int getEndExpr() {
			return endExpr;
		}

		public void setEndExpr(int endExpr) {
			this.endExpr = endExpr;
		}

		public FrameType getStartType() {
			return startType;
		}

		public void setStartType(FrameType startType) {
			this.startType = startType;
		}

		public FrameType getEndType() {
			return endType;
		}

		public void setEndType(FrameType endType) {
			this.endType = endType;
		}

		public boolean isStartFollowing() {
			return startFollowing;
		}

		public void setStartFollowing(boolean startFollowing) {
			this.startFollowing = startFollowing;
		}

		public boolean isEndFollowing() {
			return endFollowing;
		}

		public void setEndFollowing(boolean endFollowing) {
			this.endFollowing = endFollowing;
		}

		@Override
		public void acceptVisitor(LanguageObjectVisitor visitor) {
			visitor.visit(this);
		}

		@Override
		public int hashCode() {
			return HashCodeUtil.hashCode(0, range, startExpr, endExpr, startType, endType, startFollowing, endFollowing);
		}

		@Override
		public boolean equals(Object obj) {
			if (obj == this) {
				return true;
			}
			if (!(obj instanceof Frame)) {
				return false;
			}
			Frame other = (Frame) obj;
			return this.range == other.range && this.startFollowing == other.startFollowing && this.endFollowing == other.endFollowing &&
					this.startType == other.startType && this.endType == other.endType && 
					this.startExpr == other.startExpr && this.endExpr == other.endExpr;
		}
	}
	// END EXTENSION
}
