/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 *  
 */

package org.teiid.translator;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

import javax.resource.cci.Connection;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.ws.Dispatch;
import javax.xml.ws.Service;

/**
 * Simple {@link Connection} interface for web services
 */
public interface WSConnection extends Connection {
	
	public static final String ACCESS_TOKEN = "ACCESS_TOKEN";
	public static final String REFRESH_TOKEN = "REFRESH_TOKEN";
	public static final String CLIENT_ID = "CLIENT_ID";
	public static final String CLIENT_SECRET = "CLIENT_SECRET";
	public static final String ENDPOINT_TO_GET_VALID_ACCESS_TOKEN = "ENDPOINT_TO_GET_A_VALID_ACCESS_TOKEN";
	public static final String AUTH_CODE = "AUTH_CODE";
	public static final String REDIRECT_URI = "REDIRECT_URI";
	public static final String DATASOURCE_NAME = "DATASOURCE_NAME";
	public static final String INVOCATION_FAILED = "INVOCATION_FAILED";
	public static final String RESPONSE_CODE = "RESPONSE_CODE";
	
	<T> Dispatch<T> createDispatch(String binding, String endpoint, Class<T> type, Service.Mode mode);
	
	// EXTENSION
	// ISSUE: DVCORE-2218
	// FIXED BY: Salvatore Raunich
	// DATE: 2014.07.21
	<T> Dispatch<T> createDispatch(String binding, String endpoint, Class<T> type, Service.Mode mode, boolean failOnHttpError);
	// END EXTENSION
	
	public static class Util {
		
		public static String appendQueryString(String endpoint, String param) {
			return endpoint + (endpoint.indexOf('?') >= 0?'&':'?') + param;
		}
		
	    public static String httpURLEncode(String s) {
	        try {
	            return URLEncoder.encode(s, "UTF-8").replaceAll("\\+", "%20"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	        } catch (UnsupportedEncodingException e) {
	        	throw new RuntimeException(e);
	        }
	    }

		public static void closeSource(final Source source) {
			if (!(source instanceof StreamSource)) {
				return;
			}
			
			StreamSource stream = (StreamSource)source;
			try {
				if (stream.getInputStream() != null) {
					stream.getInputStream().close();
				}
			} catch (IOException e) {
			}
			try {
				if (stream.getReader() != null) {
					stream.getReader().close();
				}
			} catch (IOException e) {
			}
		}
		
	}
	
}
