/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 *
 */

package org.teiid.core.util;

import junit.framework.TestCase;

/**
 * @author Matias Szaingurten
 * @date 2023.05.12
 */
public class TestNamesUtil extends TestCase {

    public TestNamesUtil(String name) {
        super(name);
    }

    String objName = "this-is-a-name"; //$NON-NLS-1$
    String expectedCanonical = objName.toUpperCase();
    String expectedWBrackets = NamesUtil.wrap(objName);

    public void testCanonicalNameWithoutBrackets() {
        String name = objName;
        String cName = NamesUtil.getCanonicalName(name);
        assertEquals(cName, expectedCanonical);
    }

    public void testCanonicalNameWithBrackets() {
        String name = "[" + objName + "]";
        String cName = NamesUtil.getCanonicalName(name);
        assertEquals(cName, expectedCanonical);
    }

    public void testCanonicalNameWithBrackets2() {
        String name = NamesUtil.wrap(NamesUtil.wrap(objName));
        String cName = NamesUtil.getCanonicalName(name);
        assertEquals(cName, expectedCanonical);
    }

    public void testAddBrackets() {
        String name = NamesUtil.wrap(objName);
        String cName = NamesUtil.addBrackets(name);
        assertEquals(cName, expectedWBrackets);
    }

    public void testAddBrackets2() {
        String name = objName + "." + NamesUtil.wrap(objName);
        String cName = NamesUtil.addBrackets(name);
        assertEquals(cName, expectedWBrackets + "." + expectedWBrackets);
    }

    public void testAddBrackets3() {
        String name = NamesUtil.wrap(NamesUtil.wrap(NamesUtil.wrap(objName)));
        String cName = NamesUtil.addBrackets(name);
        assertEquals(cName, expectedWBrackets);
    }

    public void testNameWithBracketsInTheMiddle() {
        String proc_name_special = "foo bar_dvcore3830&#_%999*@()$+-|/[]{}<>^º`~"; //$NON-NLS-1$
        String cName = NamesUtil.addBrackets(proc_name_special);
        assertEquals(cName, NamesUtil.wrap(proc_name_special)); //$NON-NLS-1$
    }
}