/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 *  
 */
// https://issues.jboss.org/browse/TEIID-5924
package org.teiid.resource.adapter.salesforce;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Calendar;

import org.junit.Test;
import org.mockito.Mockito;
import org.teiid.translator.salesforce.execution.DataPayload;
import org.teiid.translator.salesforce.execution.DeletedResult;

import com.sforce.soap.partner.DeletedRecord;
import com.sforce.soap.partner.GetDeletedResult;
import com.sforce.soap.partner.PartnerConnection;
import com.sforce.soap.partner.sobject.SObject;

@SuppressWarnings("nls")
public class TestSalesforceConnectionImpl {
	
	@Test public void testGetDeleted() throws Exception {
		PartnerConnection pc = Mockito.mock(PartnerConnection.class);
		GetDeletedResult gdr = new GetDeletedResult();
		Calendar c = Calendar.getInstance();
		gdr.setEarliestDateAvailable(c);
		gdr.setLatestDateCovered(c);
		DeletedRecord dr = new DeletedRecord();
		dr.setDeletedDate(c);
		dr.setId("id");
		gdr.setDeletedRecords(new DeletedRecord[] {dr});
		Mockito.stub(pc.getDeleted("x", null, null)).toReturn(gdr);
		SalesforceConnectionImpl sfci = new SalesforceConnectionImpl(pc);
		DeletedResult result = sfci.getDeleted("x", null, null);
		assertEquals(1, result.getResultRecords().size());
	}

	@Test public void testUpdateValues() throws Exception {
		DataPayload payload = new DataPayload();
		payload.addField("hello", "world");
		payload.addField("null", null);
		SObject obj = SalesforceConnectionImpl.toUpdateSObject(new ArrayList<String>(), payload);
		assertArrayEquals(new String [] {"null"}, obj.getFieldsToNull());
	}

}
