/**
 * Copyright © 2012-2023 by Data Virtuality GmbH
 */
package org.teiid.translator.jdbc;

import com.datavirtuality.dv.core.interfaces.DVCaller;
import org.teiid.metadata.MetadataFactory;
import org.teiid.translator.TranslatorException;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * @author Allan Vital
 * @date 2023.07.31
 */
public class JDBCMulticatalogProcessor extends JDBCMetadataProcessor {

    @Override
    public Map<String, JDBCMetadataProcessor.TableInfo> getTables(MetadataFactory metadataFactory,
                                                                  DatabaseMetaData metadata, Connection conn, String[] tableTypes) throws SQLException, TranslatorException {

        String cleanCatalog = this.getCatalog();
        String rawParamCatalog = this.getUnescapedCatalog(metadataFactory.getModelPropertiesRaw(), cleanCatalog);
        Boolean isCatalogSingleValueAndDoesNotContainPattern = (Boolean) DVCaller.call(233, rawParamCatalog);
        if (Boolean.TRUE.equals(isCatalogSingleValueAndDoesNotContainPattern)) {
            Map<String, TableInfo> tables = super.getTables(metadataFactory, metadata, conn, tableTypes);
            if (cleanCatalog != null) {
                conn.setCatalog(cleanCatalog);
            }
            return tables;
        }
        try {
            ResultSet catalogs = metadata.getCatalogs();
            Map<String, JDBCMetadataProcessor.TableInfo> returnMap = new HashMap<>();
            while (catalogs.next()) {
                String catalogName = catalogs.getString("TABLE_CAT");
                Boolean matchPattern = (Boolean) DVCaller.call(234, catalogName, rawParamCatalog);
                if (Boolean.TRUE.equals(matchPattern)) {
                    this.setCatalog(catalogName);
                    returnMap.putAll(super.getTables(metadataFactory, metadata, conn, tableTypes));
                }
            }
            return returnMap;
        } finally {
            this.setCatalog(cleanCatalog);
        }
    }

    private String getUnescapedCatalog(Properties modelPropertiesRaw, String defaultCatalog) {
        String catalogKey = "importer.catalog";
        if (modelPropertiesRaw == null || !modelPropertiesRaw.containsKey(catalogKey)) {
            return defaultCatalog;
        }
        return modelPropertiesRaw.getProperty(catalogKey);
    }

}