/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 *  
 */

package org.teiid.translator.jdbc.oracle;

import static org.junit.Assert.*;

import org.junit.Test;

@SuppressWarnings("nls")
public class TestFormatFunctionModifier {

	OracleFormatFunctionModifier offm = new OracleFormatFunctionModifier("TO_CHAR(");
    
    public void helpTest(String expected, String format) {
    	assertTrue(OracleFormatFunctionModifier.supportsLiteral(format));
        assertEquals(expected, offm.translateFormat(format));
    }
    
    @Test public void testQuoting() {
    	// BUG: formatTimestamp and formatDate functions are not pushed down in Oracle
    	// ISSUE: DVCORE-776
    	// SOLUTION: added missing quotes when the literal is translated
    	// FIXED by Salvatore Raunich
    	// date 2013.11.06
//    	helpTest("\"a'\"\"123\"", "'a'''123");
    	helpTest("'\"a'\"\"123\"'", "'a'''123");
    	// BUG END
    }
    
    @Test public void testDay() {
    	// BUG: formatTimestamp and formatDate functions are not pushed down in Oracle
    	// ISSUE: DVCORE-776
    	// SOLUTION: added missing quotes when the literal is translated
    	// FIXED by Salvatore Raunich
    	// date 2013.11.06
//    	helpTest("DD DDD", "d D");
    	helpTest("'DD DDD'", "d D");
    	// BUG END
    }
    
    @Test public void testYear() {
    	// BUG: formatTimestamp and formatDate functions are not pushed down in Oracle
    	// ISSUE: DVCORE-776
    	// SOLUTION: added missing quotes when the literal is translated
    	// FIXED by Salvatore Raunich
    	// date 2013.11.06
//    	helpTest("YY YY YY YYYY", "y yy yyy yyyy");
    	helpTest("'YY YY YY YYYY'", "y yy yyy yyyy");
    	// BUG END
    }
    
    @Test public void testMonth() {
    	// BUG: formatTimestamp and formatDate functions are not pushed down in Oracle
    	// ISSUE: DVCORE-776
    	// SOLUTION: added missing quotes when the literal is translated
    	// FIXED by Salvatore Raunich
    	// date 2013.11.06
//    	helpTest("MM MM Mon Month", "M MM MMM MMMM");
    	helpTest("'MM MM Mon Month'", "M MM MMM MMMM");
    	// BUG END
    }
    
    @Test public void testISO() {
    	// BUG: formatTimestamp and formatDate functions are not pushed down in Oracle
    	// ISSUE: DVCORE-776
    	// SOLUTION: added missing quotes when the literal is translated
    	// FIXED by Salvatore Raunich
    	// date 2013.11.06
//    	helpTest("YYYY-MM-DD\"T\"HH24:MI:SS.FF3", "yyyy-MM-dd'T'HH:mm:ss.SSS");
    	helpTest("'YYYY-MM-DD\"T\"HH24:MI:SS.FF'", "yyyy-MM-dd'T'HH:mm:ss.SSS");
    	// BUG END
    }
    
    @Test public void testSupports() {
    	assertFalse(OracleFormatFunctionModifier.supportsLiteral("\""));
    	assertFalse(OracleFormatFunctionModifier.supportsLiteral("'"));
    	assertFalse(OracleFormatFunctionModifier.supportsLiteral("x"));
    	assertFalse(OracleFormatFunctionModifier.supportsLiteral("-ax"));
    }

}
