/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 *  
 */
// https://issues.jboss.org/browse/TEIID-4402
// https://issues.jboss.org/browse/TEIID-5923
package org.teiid.translator.salesforce;

import java.sql.Time;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import org.teiid.core.util.TimestampWithTimezone;
import org.teiid.translator.TypeFacility;

public class Util {

	public static String stripQutes(String id) {
		if((id.startsWith("'") && id.endsWith("'"))) { //$NON-NLS-1$ //$NON-NLS-2$
			id = id.substring(1,id.length()-1);
		} else if ((id.startsWith("\"") && id.endsWith("\""))) { //$NON-NLS-1$ //$NON-NLS-2$
			id = id.substring(1,id.length()-1);
		}
		return id;
	}

	/**
	 * Convert the Teiid value to one that Salesforce will correctly serialized
	 * @param value
	 * @param type
	 * @return
	 */
 	public static Object toSalesforceObjectValue(Object value, Class<?> type) {
	    if (value != null) {
		    if (type == TypeFacility.RUNTIME_TYPES.TIME) {
			    return new com.sforce.ws.types.Time(((Time) value).getTime());
		    } else if (type == TypeFacility.RUNTIME_TYPES.TIMESTAMP) {
			    Calendar cal = (Calendar) TimestampWithTimezone.getCalendar().clone();
			    cal.setTimeInMillis(((Timestamp) value).getTime());
			    return cal;
		    }
		    //all other pushdown types are directly supported
	    }
	    return value;
    }
	
	private static String timeZone;

	public static void resetTimeZone() {
		timeZone = null;
	}
	
	public static String getDefaultTimeZoneString() {
		if (timeZone == null) {
			String s = new SimpleDateFormat("Z").format(new Date(0)); //$NON-NLS-1$
			timeZone = s.substring(0, 3) + ':'  + s.substring(3, 5);
		}
		return timeZone;
	}

}
