/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 *  
 */
package org.teiid.query.metadata;

import java.util.Iterator;

import javax.resource.ResourceException;

import org.teiid.logging.LogConstants;
import org.teiid.logging.LogManager;
import org.teiid.metadata.MetadataFactory;
import org.teiid.metadata.MetadataRepository;
import org.teiid.metadata.Table;
import org.teiid.query.QueryPlugin;
import org.teiid.resource.spi.WrappedConnection;
import org.teiid.translator.ExecutionFactory;
import org.teiid.translator.TranslatorException;

// EXTENSION
// ISSUE: DVCORE-2441
// FIXED BY: Roman Pedchenko
// DATE: 2014.10.26
// ADOPTED FROM: https://issues.jboss.org/browse/TEIID-2235, 2661, 2178, 2297
// public class NativeMetadataRepository extends BaseMetadataRepository {
public class NativeMetadataRepository extends MetadataRepository {
// END EXTENSION

	@Override
	public void loadMetadata(MetadataFactory factory, ExecutionFactory executionFactory, Object connectionFactory) throws TranslatorException {
		
		if (executionFactory == null ) {
			throw new TranslatorException(QueryPlugin.Util.gs(QueryPlugin.Event.TEIID30591, factory.getName()));
		}
		
		if (connectionFactory == null && executionFactory.isSourceRequiredForMetadata()) {
			throw new TranslatorException(QueryPlugin.Event.TEIID31097, QueryPlugin.Util.gs(QueryPlugin.Event.TEIID31097));
		}
		
		Object connection = executionFactory.getConnection(connectionFactory, null);
		Object unwrapped = null;
		
		if (connection instanceof WrappedConnection) {
			try {
				unwrapped = ((WrappedConnection)connection).unwrap();
			} catch (ResourceException e) {
				if (executionFactory.isSourceRequiredForMetadata()) {
					throw new TranslatorException(QueryPlugin.Event.TEIID30477, QueryPlugin.Util.gs(QueryPlugin.Event.TEIID30477));
				}
				connection = null;
			}	
		}
		
		try {
			executionFactory.getMetadata(factory, (unwrapped == null) ? connection:unwrapped);
		} finally {
			executionFactory.closeConnection(connection, connectionFactory);
		}
    	// BUG: Tables without columns (possible e.g. in MongoDB) are not loaded and an exception is thrown. 
		// In this case the full VDB is not loaded.
		// ISSUE: DVCORE-111
		// SOLUTION: tables without columns are not loaded, a warning message is printed and the other tables
		// in the database are loaded.
		// FIXED by Salvatore Raunich
		// date 11.10.2012
		validateMetadata(factory);
		// BUG END
// EXTENSION
// ISSUE: DVCORE-2441
// FIXED BY: Roman Pedchenko
// DATE: 2014.10.26
// ADOPTED FROM: https://issues.jboss.org/browse/TEIID-2235, 2661, 2178, 2297
//     	super.loadMetadata(factory, executionFactory, connectionFactory);		
// END EXTENSION
	}
	
	// BUG: Tables without columns (possible e.g. in MongoDB) are not loaded and an exception is thrown. 
	// In this case the full VDB is not loaded.
	// ISSUE: DVCORE-111
	// SOLUTION: tables without columns are not loaded, a warning message is printed and the other tables
	// in the database are loaded.
	// FIXED by Salvatore Raunich
	// date 11.10.2012
	private void validateMetadata(MetadataFactory schema) throws TranslatorException {
		for (Iterator<Table> it = schema.getSchema().getTables().values().iterator(); it.hasNext();) {
			Table t = (Table) it.next();
			if ((t.getColumns() == null || t.getColumns().size() == 0) && t.getTableType() != Table.Type.View) {
				it.remove();
				LogManager.logWarning(LogConstants.CTX_RUNTIME, "Can't load this table because it has no columns: " + t.getFullName());
			}
		}
	}	
	// BUG END
}
