// EXTENSION
// ISSUE: DVCORE-2441
// FIXED BY: Roman Pedchenko
// DATE: 2014.10.23
// ADOPTED FROM: https://issues.jboss.org/browse/TEIID-247, 2166, 2151, 2234, 2581, 2571, 2369, 1386, 2226, 2233, 2224, 2225, 2257
/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 *  
 */
// https://issues.jboss.org/browse/TEIID-2402

package org.teiid.query.parser;

import com.datavirtuality.dv.core.interfaces.RecorderStream;

import java.io.IOException;

/**
 * A customized TokenManager to keep a token history and to treat lexical errors as invalid single character tokens
 */
class TeiidSQLParserTokenManager extends SQLParserTokenManager implements RecorderStream {

	static final int INVALID_TOKEN = 10; // no constant with that number in SQLParserConstants
	int tokenCount = 0;
	Token head;
	
	public TeiidSQLParserTokenManager(JavaCharStream stream) {
		super(stream);
	}
	
	void reinit() {
		tokenCount = 0;
		head = null;
	}

	public void startRecord(int start) {
		if (input_stream instanceof RecorderStream)
			((RecorderStream) input_stream).startRecord(start);
	}

	public String getRecorded(int end) {
		if (input_stream instanceof RecorderStream)
			return ((RecorderStream) input_stream).getRecorded(end);
		return "";
	}

	@Override
	public Token getNextToken() {
		try {
			Token t = super.getNextToken();
			//if we're in the default lex state, keep track of a history of tokens
			//this logic is not perfect as deep lookaheads can ruin the positioning
			if (tokenCount == 0) {
				head = t;
			}
			tokenCount++;
			if (tokenCount > 10) {
				head = head.next;
			}
			return t;
		} catch (TokenMgrError err) {
			Token t = new Token();
			t.kind = INVALID_TOKEN;
			t.invalid = true;
			t.beginColumn = this.input_stream.getBeginColumn();
			t.beginLine = this.input_stream.getBeginLine();
			t.endColumn = t.beginColumn;
			t.endLine = t.beginLine;
			t.image = this.input_stream.GetImage().substring(0, 1);
			try {
				//mark the char a consumed
				this.input_stream.readChar();
			} catch (IOException ignored) {
			}
			return t;
		}
	}

}
// END EXTENSION
