/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 *  
 */
// https://issues.jboss.org/browse/TEIID-3053
// https://issues.jboss.org/browse/TEIID-4509

/*
 */
package org.teiid.query.processor.proc;

import org.teiid.client.plan.PlanNode;
import org.teiid.core.TeiidComponentException;
import org.teiid.core.TeiidProcessingException;
import org.teiid.dqp.internal.process.AuthorizationValidator.CommandType;
import org.teiid.query.metadata.QueryMetadataInterface;
import org.teiid.query.processor.ProcessorPlan;
import org.teiid.query.sql.lang.Command;
import org.teiid.query.sql.symbol.ElementSymbol;
import org.teiid.query.util.CommandContext;

import java.util.Map;

import static org.teiid.query.analysis.AnalysisRecord.PROP_RESULT_SET;
import static org.teiid.query.analysis.AnalysisRecord.PROP_SQL;


/**
 */
public class CreateCursorResultSetInstruction extends ProgramInstruction {
	public enum Mode {
		UPDATE,
		HOLD,
		NOHOLD
	}

    protected Command command;
    private final QueryMetadataInterface metadata;
    protected String rsName;
    protected ProcessorPlan plan;
    private final Mode mode;
    private Map<ElementSymbol, ElementSymbol> procAssignments;
    private boolean usesLocalTemp;
    
    public CreateCursorResultSetInstruction(String rsName, Command command, QueryMetadataInterface metadata, ProcessorPlan plan, Mode mode){
        this.rsName = rsName;
        this.command = command;
        this.metadata = metadata;
        this.plan = plan;
        this.mode = mode;
    }
    
    public void setProcAssignments(
			Map<ElementSymbol, ElementSymbol> procAssignments) {
		this.procAssignments = procAssignments;
	}
    
    public void process(ProcedurePlan procEnv)
        throws TeiidComponentException, TeiidProcessingException {
        if (command != null && metadata != null) {
            CommandContext context = procEnv.getContext();
            if (context != null) {
                boolean isAdmin = context.getDQPWorkContext() != null && context.getDQPWorkContext().isAdmin();
                if (procEnv.isValidateAccess() && !isAdmin && context.getAuthorizationValidator() != null) {
                    context.getAuthorizationValidator().validate(new String[]{command.toString()}, command, metadata,
                            context, CommandType.USER);
                }
            }
        }
        procEnv.executePlan(plan, rsName, procAssignments, mode, usesLocalTemp);
    }

    /**
     * Returns a deep clone
     */
    public CreateCursorResultSetInstruction clone() {
        ProcessorPlan clonedPlan = this.plan.clone();
        CreateCursorResultSetInstruction clone = new CreateCursorResultSetInstruction(this.rsName, command, metadata, clonedPlan, mode);
        clone.setProcAssignments(procAssignments);
        clone.usesLocalTemp = true;
        return clone;
    }
    
    public String toString(){
        return "CREATE CURSOR RESULTSET INSTRUCTION - " + rsName; //$NON-NLS-1$
    }
    
    public PlanNode getDescriptionProperties() {
        PlanNode props = new PlanNode("CREATE CURSOR"); //$NON-NLS-1$
        props.addProperty(PROP_RESULT_SET, this.rsName);
        props.addProperty(PROP_SQL, this.plan.getDescriptionProperties());
        return props;
    }
    
    public ProcessorPlan getCommand() { //Defect 13291 - added method to support changes to ProcedurePlan
        return plan;
    }
    
	public Mode getMode() {
		return mode;
	}

	public void setUsesLocalTemp(boolean b) {
		this.usesLocalTemp = b;
	}
	
	@Override
	public Boolean requiresTransaction(boolean transactionalReads) {
		return plan.requiresTransaction(transactionalReads);
	}
}
