/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 *  
 */

package org.teiid.query.resolver;

import java.util.ArrayList;
import java.util.Collection;

import org.teiid.query.metadata.TempMetadataID;
import org.teiid.query.sql.LanguageObject;
import org.teiid.query.sql.LanguageVisitor;
import org.teiid.query.sql.navigator.DeepPreOrderNavigator;
import org.teiid.query.sql.symbol.ElementSymbol;
import org.teiid.query.sql.symbol.GroupSymbol;
import org.teiid.query.sql.symbol.Symbol;


/**
 * Used to verify that all symbols in a LanguageObject were resolved
 * with respect to runtime metadata - also that they aren't resolved
 * to TemporaryMetadatID's.
 */
public class CheckNoTempMetadataIDsVisitor extends LanguageVisitor {

    // EXTENSION
    // ISSUE: DVCORE-2441
    // FIXED BY: Roman Pedchenko
    // DATE: 2014.10.20
    // ADOPTED FROM: https://issues.jboss.org/browse/TEIID-2111
    // private Collection symbolsWithTempMetadataIDs;
    // private Collection ignoreTheseSymbols;
    private Collection<Symbol> symbolsWithTempMetadataIDs;
    // END EXTENSION
    
    /**
     * By default, this visitor deeply traverses all commands, and there are
     * no symbols to ignore
     */
	public CheckNoTempMetadataIDsVisitor() { 
        // EXTENSION
        // ISSUE: DVCORE-2441
        // FIXED BY: Roman Pedchenko
        // DATE: 2014.10.20
        // ADOPTED FROM: https://issues.jboss.org/browse/TEIID-2111
        // symbolsWithTempMetadataIDs = new ArrayList();    
        // this.ignoreTheseSymbols = Collections.EMPTY_LIST;
        symbolsWithTempMetadataIDs = new ArrayList<Symbol>();    
        // END EXTENSION
    }
    
    /**
     * Get the Collection of any unresolved symbols
     * @return Collection of any unresolved Symbols; may
     * be empty but never null
     */
    // EXTENSION
    // ISSUE: DVCORE-2441
    // FIXED BY: Roman Pedchenko
    // DATE: 2014.10.20
    // ADOPTED FROM: https://issues.jboss.org/browse/TEIID-2111
    // public Collection getSymbols(){
    public Collection<Symbol> getSymbols(){
    // END EXTENSION
        return this.symbolsWithTempMetadataIDs;
    }

    /**
     * By default, this visitor deeply traverses all commands, and there are
     * no symbols to ignore
     */
    // EXTENSION
    // ISSUE: DVCORE-2441
    // FIXED BY: Roman Pedchenko
    // DATE: 2014.10.20
    // ADOPTED FROM: https://issues.jboss.org/browse/TEIID-2111
    // public static final Collection checkSymbols(LanguageObject obj){
    public static final Collection<Symbol> checkSymbols(LanguageObject obj){
    // END EXTENSION
        CheckNoTempMetadataIDsVisitor visitor = new CheckNoTempMetadataIDsVisitor();
        DeepPreOrderNavigator.doVisit(obj, visitor);
        return visitor.getSymbols();
    }

    // visitor methods
    
    public void visit(ElementSymbol obj) {
        // EXTENSION
        // ISSUE: DVCORE-2441
        // FIXED BY: Roman Pedchenko
        // DATE: 2014.10.20
        // ADOPTED FROM: https://issues.jboss.org/browse/TEIID-2111
        // if (obj.getMetadataID() instanceof TempMetadataID && !(this.ignoreTheseSymbols.contains(obj))){
        if (obj.getMetadataID() instanceof TempMetadataID){
        // END EXTENSION
            this.symbolsWithTempMetadataIDs.add(obj);
        }
    }

    public void visit(GroupSymbol obj) {
        // EXTENSION
        // ISSUE: DVCORE-2441
        // FIXED BY: Roman Pedchenko
        // DATE: 2014.10.20
        // ADOPTED FROM: https://issues.jboss.org/browse/TEIID-2111
        // if (obj.getMetadataID() instanceof TempMetadataID && !(this.ignoreTheseSymbols.contains(obj))){
        if (obj.getMetadataID() instanceof TempMetadataID){
        // END EXTENSION
            this.symbolsWithTempMetadataIDs.add(obj);
        }
    }

}
