/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 *  
 */

package org.teiid.query.sql.proc;

import org.teiid.core.util.UnitTestUtil;
import org.teiid.query.sql.proc.RaiseStatement;
import org.teiid.query.sql.symbol.Constant;

import junit.framework.*;

/**
 *
 * @author gchadalavadaDec 11, 2002
 */
public class TestRaiseErrorStatement  extends TestCase {

	/**
	 * Constructor for TestAssignmentStatement.
	 */
	public TestRaiseErrorStatement(String name) { 
		super(name);
	}
	
	// ################################## TEST HELPERS ################################	

	// EXTENSION
	// ISSUE: DVCORE-2441
	// FIXED BY: Roman Pedchenko
	// DATE: 2014.10.23
	// ADOPTED FROM: https://issues.jboss.org/browse/TEIID-247, 2166, 2151, 2234, 2581, 2571, 2369, 1386, 2226, 2233, 2224, 2225, 2257
	// public static final RaiseErrorStatement sample1() { 
	// 	return new RaiseErrorStatement(new Constant("a")); //$NON-NLS-1$
	public static final RaiseStatement sample1() { 
		return new RaiseStatement(new Constant("a")); //$NON-NLS-1$
	// END EXTENSION
	}
	
	// EXTENSION
	// ISSUE: DVCORE-2441
	// FIXED BY: Roman Pedchenko
	// DATE: 2014.10.23
	// ADOPTED FROM: https://issues.jboss.org/browse/TEIID-247, 2166, 2151, 2234, 2581, 2571, 2369, 1386, 2226, 2233, 2224, 2225, 2257
	// public static final RaiseErrorStatement sample2() {
	// 	return new RaiseErrorStatement(new Constant("b")); //$NON-NLS-1$
	public static final RaiseStatement sample2() {
		return new RaiseStatement(new Constant("b")); //$NON-NLS-1$
	// END EXTENSION
	}
	
	// ################################## ACTUAL TESTS ################################	
	
	public void testSelfEquivalence(){
		// EXTENSION
		// ISSUE: DVCORE-2441
		// FIXED BY: Roman Pedchenko
		// DATE: 2014.10.23
		// ADOPTED FROM: https://issues.jboss.org/browse/TEIID-247, 2166, 2151, 2234, 2581, 2571, 2369, 1386, 2226, 2233, 2224, 2225, 2257
		// RaiseErrorStatement s1 = sample1();
		RaiseStatement s1 = sample1();
		// END EXTENSION
		int equals = 0;
		UnitTestUtil.helpTestEquivalence(equals, s1, s1);
	}

	public void testEquivalence(){
		// EXTENSION
		// ISSUE: DVCORE-2441
		// FIXED BY: Roman Pedchenko
		// DATE: 2014.10.23
		// ADOPTED FROM: https://issues.jboss.org/browse/TEIID-247, 2166, 2151, 2234, 2581, 2571, 2369, 1386, 2226, 2233, 2224, 2225, 2257
		// RaiseErrorStatement s1 = sample1();
		// RaiseErrorStatement s1a = sample1();
		RaiseStatement s1 = sample1();
		RaiseStatement s1a = sample1();
		// END EXTENSION
		int equals = 0;
		UnitTestUtil.helpTestEquivalence(equals, s1, s1a);
	}
	
	public void testNonEquivalence(){
		// EXTENSION
		// ISSUE: DVCORE-2441
		// FIXED BY: Roman Pedchenko
		// DATE: 2014.10.23
		// ADOPTED FROM: https://issues.jboss.org/browse/TEIID-247, 2166, 2151, 2234, 2581, 2571, 2369, 1386, 2226, 2233, 2224, 2225, 2257
		// RaiseErrorStatement s1 = sample1();
		// RaiseErrorStatement s2 = sample2();
		RaiseStatement s1 = sample1();
		RaiseStatement s2 = sample2();
		// END EXTENSION
		int equals = -1;
		UnitTestUtil.helpTestEquivalence(equals, s1, s2);
	}
}
