/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * Modified by
 * @author  Data Virtuality GmbH
 * @date    $Date$
 *  
 */
package org.teiid.deployers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;
import java.util.TreeMap;

import org.teiid.core.util.StringUtil;
import org.teiid.metadata.FunctionMethod;
import org.teiid.query.function.UDFSource;


public class UDFMetaData {
	// EXTENSION
	// ISSUE: DVCORE-2441
	// FIXED BY: Roman Pedchenko
	// DATE: 2014.10.23
	// ADOPTED FROM: https://issues.jboss.org/browse/TEIID-247, 2166, 2151, 2234, 2581, 2571, 2369, 1386, 2226, 2233, 2224, 2225, 2257
	// protected HashMap<String, Collection <FunctionMethod>> methods = new HashMap<String, Collection<FunctionMethod>>();	
	protected TreeMap<String, UDFSource> methods = new TreeMap<String, UDFSource>(StringUtil.CASE_INSENSITIVE_ORDER);	
	// END EXTENSION
	private ClassLoader classLoader;
	// EXTENSION
	// ISSUE: DVCORE-2441
	// FIXED BY: Roman Pedchenko
	// DATE: 2014.10.23
	// ADOPTED FROM: https://issues.jboss.org/browse/TEIID-247, 2166, 2151, 2234, 2581, 2571, 2369, 1386, 2226, 2233, 2224, 2225, 2257
	// public Map<String, Collection <FunctionMethod>> getFunctions(){
	
	public Map<String, UDFSource> getFunctions(){
	// END EXTENSION
		return this.methods;
	}

	public void addFunctions(String name, Collection <FunctionMethod> funcs){
		if (funcs.isEmpty()) {
			return;
		}
		// EXTENSION
		// ISSUE: DVCORE-2441
		// FIXED BY: Roman Pedchenko
		// DATE: 2014.10.23
		// ADOPTED FROM: https://issues.jboss.org/browse/TEIID-247, 2166, 2151, 2234, 2581, 2571, 2369, 1386, 2226, 2233, 2224, 2225, 2257
		// Collection <FunctionMethod> old = this.methods.put(name, funcs);
		// if (old != null) {
		// 	ArrayList<FunctionMethod> combined = new ArrayList<FunctionMethod>(old);
		// 	combined.addAll(funcs);
		// 	this.methods.put(name, combined);
		UDFSource udfSource = this.methods.get(name);
		if (udfSource != null) {
			//this is ambiguous about as to what classloader to use, but we assume the first is good and that the user will have set 
			//the Java method if that's not the case
			ArrayList<FunctionMethod> allMethods = new ArrayList<FunctionMethod>(udfSource.getFunctionMethods());
			allMethods.addAll(funcs);
			ClassLoader cl = udfSource.getClassLoader();
			udfSource = new UDFSource(allMethods);
			udfSource.setClassLoader(cl);
		} else {
			udfSource = new UDFSource(funcs);
			udfSource.setClassLoader(classLoader);
		// END EXTENSION
		}
		// EXTENSION
		// ISSUE: DVCORE-2441
		// FIXED BY: Roman Pedchenko
		// DATE: 2014.10.23
		// ADOPTED FROM: https://issues.jboss.org/browse/TEIID-247, 2166, 2151, 2234, 2581, 2571, 2369, 1386, 2226, 2233, 2224, 2225, 2257
		this.methods.put(name, udfSource);
		// END EXTENSION
	}
	
	public void addFunctions(UDFMetaData funcs){
		// EXTENSION
		// ISSUE: DVCORE-2441
		// FIXED BY: Roman Pedchenko
		// DATE: 2014.10.23
		// ADOPTED FROM: https://issues.jboss.org/browse/TEIID-247, 2166, 2151, 2234, 2581, 2571, 2369, 1386, 2226, 2233, 2224, 2225, 2257
		// for (Map.Entry<String, Collection<FunctionMethod>> entry : funcs.getFunctions().entrySet()) {
		// 	addFunctions(entry.getKey(), entry.getValue());
		// }
		this.methods.putAll(funcs.methods);
		this.classLoader = funcs.classLoader;
		// END EXTENSION
	}

	public void setFunctionClassLoader(ClassLoader functionClassLoader) {
		// EXTENSION
		// ISSUE: DVCORE-2441
		// FIXED BY: Roman Pedchenko
		// DATE: 2014.10.23
		// ADOPTED FROM: https://issues.jboss.org/browse/TEIID-247, 2166, 2151, 2234, 2581, 2571, 2369, 1386, 2226, 2233, 2224, 2225, 2257
		for (UDFSource udf : methods.values()) {
			udf.setClassLoader(functionClassLoader);
		}
		// END EXTENSION
		this.classLoader = functionClassLoader;
	}
	
	public ClassLoader getClassLoader() {
		return classLoader;
	}
}
